# -*- coding: utf-8 -*-
import re, time
import numpy as np

import zhinst.ziPython as ziPython
import zhinst.utils as utils

def run_example(port=None,amplitude=0.25,do_plot=False):
    """
    Record demodulator sample data when demod R violates a positive edge
    threshold using ziPython's ziDAQRecorder module. The ziDAQRecorder module
    implements software triggering analogously to the types of triggering
    found in spectroscopes.

    Additional configuration: Connect signal output 1 to signal input 1 with a
    BNC cable.
    """
    # create an instance of ziDAQServer: connect to the server
    daq = utils.autoConnect(port)
    # find an attached instrument
    device = utils.autoDetect(daq)

    # find out whether the device is an HF2 or a UHF
    devtype = daq.getByte('/%s/features/devtype' %device)
    options = daq.getByte('/%s/features/options' %device)

    # configure the device for recording
    out_c = 0
    in_c = 0
    demod_c = 0
    osc_c = 0
    demod_rate = 10e3
    if re.match('.*LI',devtype) and not re.match('.*MF',options):
        out_mixer_c = 6
    else:
        out_mixer_c = 0

    # create a base configuration: disable all outputs, demods and scopes
    general_setting = [
         ['/%s/demods/*/rate' %device, 0],
         ['/%s/demods/*/trigger' %device, 0],
         ['/%s/sigouts/*/enables/*' %device, 0]
    ]
    # if the device is a UHF additionally disable all demodulators
    if re.match('UHF',devtype):
        general_setting.append( ['/%s/demods/*/enable' %device, 0] )
        general_setting.append( ['/%s/scopes/*/enable' %device, 0] )
    elif re.match('HF2',devtype):
        general_setting.append( ['/%s/scopes/*/trigchannel' %device, -1 ])
    daq.set(general_setting)

    # now set the settings relevant to this experiment
    exp_setting = [
        ['/%s/sigins/%d/imp50' %(device,in_c), 0],
        ['/%s/sigins/%d/ac'    %(device,in_c), 0],
        ['/%s/sigins/%d/range' %(device,in_c), 2*amplitude],
        
        ['/%s/demods/%d/order'        %(device,demod_c), 4],
        ['/%s/demods/%d/timeconstant' %(device,demod_c), 0.005],
        ['/%s/demods/%d/rate'         %(device,demod_c), demod_rate],
        ['/%s/demods/%d/harmonic'     %(device,demod_c), 1],
        ['/%s/demods/%d/phaseshift'   %(device,demod_c), 0],

        ['/%s/sigouts/%d/on'            %(device,out_c), 1],
        ['/%s/sigouts/%d/range'         %(device,out_c), 1],
        ['/%s/sigouts/%d/enables/%d'    %(device,out_c,out_mixer_c), 1],
        ['/%s/sigouts/%d/amplitudes/%d' %(device,out_c,out_mixer_c), amplitude]
    ]
    # device dependent configuration
    if re.match('UHF',devtype):
        exp_setting.append( ['/%s/demods/%d/enable' %(device,demod_c), 1], )
        if re.match('.*MF',options):
            exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
            exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
    elif re.match('HF2',devtype):
        exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
        exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
        exp_setting.append( ['/%s/sigins/%d/diff' %(device,in_c), 0] )
        exp_setting.append( ['/%s/sigouts/%d/add'  %(device,out_c), 0] )
    daq.set(exp_setting);
    # wait to get a settled lowpass filter
    time.sleep(2)

    # create an instance of the recorder class
    record_buffer_length = 0.300 # this should be larger than trigger_duration
    record_timeout = 1000
    record_flag = 0
    trigger = daq.record( record_buffer_length, record_timeout, record_flag )
    
    # below we will (somewhat artificially) generate num_pulses pulses on the
    # signal outputs in order to demonstrate the triggering functionality
    sigouts_high = 1.5*amplitude
    sigouts_low = 1.0*amplitude
    num_pulses = 20
    # scale by 1/sqrt(2) due to RMS value of demod R
    trigger_highlevel = 0.5*(sigouts_low + sigouts_high)/np.sqrt(2)
    trigger_lowlevel = 0.95*trigger_highlevel

    # use an edge trigger
    trigger.set('trigger/0/type', 1) # 1 = edge
    # trigger on demod R
    trigger.set('trigger/0/source', 2)
    trigger.set('trigger/0/path', '/%s/demods/%d/sample' %(device,demod_c))
    # trigger on the positive edge
    trigger.set('trigger/0/edge', 1) # 1 = positive
    # set the trigger level
    print "Setting trigger/0/highlevel to %.3f" %trigger_highlevel 
    print "Setting trigger/0/lowlevel to %.3f" %trigger_lowlevel 
    trigger.set('trigger/0/highlevel', trigger_highlevel)
    trigger.set('trigger/0/lowlevel', trigger_lowlevel)
    # number of times to trigger
    trigger_count = 10
    trigger.set('trigger/0/count', trigger_count)
    trigger.set('trigger/0/holdoff/count', 0)
    trigger.set('trigger/0/holdoff/time', 0.100)
    trigger_delay = -0.020
    trigger.set('trigger/0/delay', trigger_delay)
    # the length of time to record each time we trigger
    trigger_duration = 0.180
    trigger.set('trigger/0/duration', trigger_duration)
    trigger.set('trigger/0/retrigger', 0)
    trigger.set('trigger/historylength', 100)

    trigger.subscribe('/%s/demods/%d/sample' %(device,demod_c))
    
    # start the recorder's thread
    trigger.execute()
    time.sleep(0.5)

    # generate some pulses on the signal outputs, this is purely to
    # demonstrate triggering on a demodulator signal and is not necessary to
    # configure the module
    for i in range(0,num_pulses):
        daq.setDouble( '/%s/sigouts/%d/amplitudes/%d' %(device,out_c,out_mixer_c), sigouts_low*(1+0.05*np.random.uniform(-1,1,1)[0]) )
        time.sleep(0.2+np.random.uniform(-0.01,0.01,1))
        daq.setDouble( '/%s/sigouts/%d/amplitudes/%d' %(device,out_c,out_mixer_c), sigouts_high*(1+0.05*np.random.uniform(-1,1,1)[0]) )
        time.sleep(0.1+np.random.uniform(-0.01,0.01,1))
        # check and display recorder progress
        progress = trigger.progress()
        print "Recorder progress (acquiring %d triggers): %.2f%%\r" %(trigger_count,100*progress),
        # check whether the recorder has finished
        if trigger.finished():
            break
    print ""
    daq.setDouble( '/%s/sigouts/%d/amplitudes/%d' %(device,out_c,out_mixer_c), sigouts_low )
    time.sleep(0.5)

    # read the recorder data. this command can also be executed whilst
    # triggering (before finished() is True), in this case data recorded up to
    # that time point is returned. we still need to issue read() at the end to
    # fetch the rest.
    data = trigger.read()

    # stop the recorder
    trigger.finish()
    # stop the recorder thread and clear the memory
    trigger.clear()

    # check the dictionary returned is non-empty
    assert data, "read() returned an empty data dictionary, did you subscribe to any paths?"
    # note: data could be empty if no data arrived, e.g., if the demods were
    # disabled or had rate 0
    assert device in data, "data dictionary has no key '%s'" %(device)
    samples = data[device]['demods'][str(demod_c)]['sample']

    if do_plot and len(samples) > 0:
        import matplotlib
        import matplotlib.pyplot as plt

        # get the sample rate of the device's ADCs
        if re.match('HF2',devtype):
            # the sample rate of the HF2's ADCs
            clockbase = 210.0e6
        elif re.match('UHF',devtype):
            clockbase = daq.getInt('/%s/clockbase' %device)
            # ensure float to avoid problems with integer division
            clockbase = float(clockbase)

        fig = plt.figure()
        ax = plt.subplot(1,1,1)
        plt.hold(True)
        p1 = plt.axvline(0.0,linewidth=2,linestyle='--',color='k',label='trigger_delay')
        p2 = plt.axhline(trigger_lowlevel,linewidth=2,linestyle='-.',color='b',label='trigger_lowlevel')
        p3 = plt.axhline(trigger_highlevel,linewidth=2,linestyle='--',color='b',label='trigger_highlevel')
        handles, labels = ax.get_legend_handles_labels()
        plt.legend(handles,labels)
        for i in range(0,len(samples)):
            t = (samples[i]['timestamp']-samples[i]['timestamp'][0])/clockbase + trigger_delay
            R = np.sqrt(samples[i]['x']**2 + samples[i]['y']**2)
            plt.plot(t,R)
        plt.grid(True)
        plt.title('Recorded %d shots of duration %.3f seconds'%(len(samples),trigger_duration))
        plt.xlabel('Time ($s$)')
        plt.ylabel('Amplitude ($V_\mathrm{RMS}$)')
        plt.ylim([0.5*amplitude,1.5*amplitude])
        plt.draw()
        plt.show()

    return samples

def test_example(port):
    """
    Run the example and perform a simple test.
    """
    samples = run_example(port)
    assert len(samples) > 0

if __name__ == '__main__':
    samples = run_example()

