# -*- coding: utf-8 -*-
import re, time
import numpy as np

import zhinst.ziPython as ziPython
import zhinst.utils as utils

def run_example(port=None,amplitude=0.1,do_plot=False):
    """
    Perform a zoom FFT using ziPython's ziDAQZoomFFT module.

    Additional configuration: Connect signal output 1 to signal input 1 with a
    BNC cable.
    """
    # create an instance of ziDAQServer: connect to the server
    daq = utils.autoConnect(port)
    # find an attached instrument
    device = utils.autoDetect(daq)

    # find out whether the device is an HF2 or a UHF
    devtype = daq.getByte('/%s/features/devtype' %device)
    options = daq.getByte('/%s/features/options' %device)

    # configure the device
    out_c = 0
    in_c = 0
    demod_c = 0
    osc_c = 0
    demod_rate = 10e3
    if re.match('.*LI',devtype) and not re.match('.*MF',options):
        out_mixer_c = 6
    else:
        out_mixer_c = 0

    # create a base configuration: disable all outputs, demods and scopes
    general_setting = [
         ['/%s/demods/*/rate' %device, 0],
         ['/%s/demods/*/trigger' %device, 0],
         ['/%s/sigouts/*/enables/*' %device, 0]
    ]

    # if the device is a UHF additionally disable all demodulators
    if re.match('UHF',devtype):
        general_setting.append( ['/%s/demods/*/enable' %device, 0] )
        general_setting.append( ['/%s/scopes/*/enable' %device, 0] )
    elif re.match('HF2',devtype):
        general_setting.append( ['/%s/scopes/*/trigchannel' %device, -1 ])
    daq.set(general_setting)

    # now set the settings relevant to this experiment
    exp_setting = [
        ['/%s/sigins/%d/imp50' %(device,in_c), 1],
        ['/%s/sigins/%d/ac'    %(device,in_c), 0],
        ['/%s/sigins/%d/range' %(device,in_c), 2*amplitude],
        
        ['/%s/demods/%d/order'        %(device,demod_c), 4],
        ['/%s/demods/%d/rate'         %(device,demod_c), demod_rate],
        ['/%s/demods/%d/timeconstant' %(device,demod_c), 8e-5],
        ['/%s/demods/%d/harmonic'     %(device,demod_c), 1],
        ['/%s/demods/%d/phaseshift'   %(device,demod_c), 0],

        ['/%s/oscs/%d/freq' %(device,demod_c), 1.5e6],

        ['/%s/sigouts/%d/on'            %(device,out_c), 1],
        ['/%s/sigouts/%d/range'         %(device,out_c), 1],
        ['/%s/sigouts/%d/enables/%d'    %(device,out_c,out_mixer_c), 1],
        ['/%s/sigouts/%d/amplitudes/%d' %(device,out_c,out_mixer_c), amplitude]
    ]
    # device dependent configuration
    if re.match('UHF',devtype):
        exp_setting.append( ['/%s/demods/%d/enable' %(device,demod_c), 1], )
        if re.match('.*MF',options):
            exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
            exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
    elif re.match('HF2',devtype):
        exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
        exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
        exp_setting.append( ['/%s/sigins/%d/diff' %(device,in_c), 0] )
        exp_setting.append( ['/%s/sigouts/%d/add'  %(device,out_c), 0] )
    daq.set(exp_setting);
    # wait to get a settled lowpass filter
    time.sleep(1.0)

    # create a thread for an instance of the zoomFFT class
    timeout = 500
    zoomFFT = daq.zoomFFT(timeout)
    # set zoomFFT parameters
    # the device on which the zoomFFT will be performed
    zoomFFT.set('zoomFFT/device', device)
    # select standard FFT(x + iy)
    zoomFFT.set('zoomFFT/mode', 0)
    # disable overlap mode
    zoomFFT.set('zoomFFT/overlap', 0)
    # return absolute frequencey
    zoomFFT.set('zoomFFT/absolute', 1)
    # number of lines 2^bits
    zoomFFT.set('zoomFFT/bit', 16)
    # number of zoomFFT's to perform
    loopcount = 1
    zoomFFT.set('zoomFFT/loopcount',loopcount)

    # subscribe to the nodes from which data will be recorded
    path = '/%s/demods/%d/sample' %(device,demod_c)
    zoomFFT.subscribe(path)

    # start the zoomFFT
    zoomFFT.execute()

    start = time.time()
    timeout = 30 # [s]
    print "Will perform %d zoomFFTs" %loopcount
    while not zoomFFT.finished():
        time.sleep(0.2)
        # please note: progress() and finish() works for first zoomFFT, but
        # it's a known issue that it doesn't update for subsequent zoomFFTs
        p = zoomFFT.progress()
        print "Individual zoomFFT %.2f complete.\r" %(100*p),
        ## here we could read intermediate data via 
        # data = zoomFFT.read()...
        ## and process it
        # if device in data:
        # ...
        if (time.time() - start) > timeout :
            # if for some reason the zoomFFT is blocking force the end of the
            # measurement
            print "\nzoomFFT still not finished, forcing finish..."
            zoomFFT.finish()
    print ""

    # read the zoomFFT data. this command can also be executed whilst the
    # zoomFFT is still being calculated (before finished() is True), in this
    # case zoomFFT data up to that time point is returned. it's still
    # necessary to issue read() at the end to fetch the rest.
    data = zoomFFT.read()
    zoomFFT.unsubscribe(path)
    
    # stop the sweeper thread and clear the memory
    zoomFFT.clear()

    # check the dictionary returned is non-empty
    assert data, "read() returned an empty data dictionary, did you subscribe to any paths?"
    # note: data could be empty if no data arrived, e.g., if the demods were
    # disabled or had rate 0
    assert device in data, "data dictionary has no key '%s'" %(device)
    samples = data[device]['demods'][str(demod_c)]['sample']
    num_zoomFFT = len(samples)
    print "sample contains %d zoomFFTs" %num_zoomFFT
    # please note: the "[0][0]" indexing is known issue to be fixed in an
    # upcoming release
    print "Number of lines in first zoomFFT: %d" %len(samples[0][0]['grid'])

    if do_plot:
        import matplotlib
        import matplotlib.pyplot as plt

        plot_idx = num_zoomFFT - 1
        print "Will plot result of zoomFFT sample[%d]" %plot_idx
        frequencies = samples[plot_idx][0]['grid']
        r = samples[plot_idx][0]['r']
        filter_data = samples[plot_idx][0]['filter']

        fig = plt.figure()

        plt.subplot(211)
        plt.hold(True)
        plt.title('Results of zoomFFT')
        plt.plot( frequencies, 20*np.log10(r*2*np.sqrt(2)/amplitude) )
        plt.grid(True)
        plt.xlabel('Frequency (Hz)')
        plt.ylabel('Amplitude (dBV)')
        plt.autoscale(True,'both',True)

        plt.subplot(212)
        plt.hold(True)
        plt.plot( frequencies, 20*np.log10((r/filter_data)*2*np.sqrt(2)/amplitude) )
        plt.grid(True)
        plt.xlabel('Frequency (Hz)')
        plt.ylabel('Amplitude (dBV) scaled with Filter')
        plt.autoscale(True,'both',True)

        plt.draw()
        plt.show()

    return samples

def test_example(port):
    """
    Run the example and perform a simple test.
    """
    samples = run_example(port)
    assert len(samples) > 0

if __name__ == '__main__':
    samples = run_example(8005,0.1,True)
